<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;

class ObsCtrlExped extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName;
    protected $util;
    protected $obsctrlexpedService;

    public function __construct(
        \Teknisa\GER\Services\CrudService $crudService, $parameterBag,
        \Teknisa\COZ\Services\ObsCtrlExpedService $obsctrlexpedService, $util
    ) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->util = $util;
        $this->obsctrlexpedService = $obsctrlexpedService;
    }

    public function obsctrlexpConsolidado(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = "obsctrlexpconsolidado";
//        $this->parameterBagColumns = array('NRORG');

        $this->find($request, $response);
    }

    public function pontoEntregaAgregada(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = "ponto_entrega_agregada";
        $this->parameterBagColumns = array('NRORG', 'FILISERV', 'FILIAIS');

        $this->find($request, $response);
    }

    private function retornaPontoEntrega($nrorg, $servico, $filial, $response){
        $filterCriteria = new DataSource\FilterCriteria('ponto_entrega_agregada');
        $filterCriteria->addCondition('NRORG', $nrorg);
        $filterCriteria->addCondition('FILISERV', $servico);
        $filterCriteria->addCondition('FILIAIS', $filial);
        $newRequest = new DTO\Request\Filter($filterCriteria, '', '', '');
        $this->pontoEntregaAgregada($newRequest, $response);
        return $response->getDataSets()[0]->getRows();
    }

    private function retornaObsPorPontoData(
        $nrorg, $central, $filial, $servico, $cliente, $destino,
        $diaSemana, $vigencia, $response
    ){
        $filterCriteria = new DataSource\FilterCriteria('obsctrlexped');
        $filterCriteria->addCondition('NRORG', $nrorg);
        $filterCriteria->addCondition('CDCENTRAL', $central);
        $filterCriteria->addCondition('CDFILIAL', $filial);
        $filterCriteria->addCondition('CDSERVFILI', $servico);
        $filterCriteria->addCondition('CDCLIENTE', $cliente);
        $filterCriteria->addCondition('CDDESTFILI', $destino);
        $filterCriteria->addCondition('NRDIASEMANA', $diaSemana);
        $filterCriteria->addCondition('DTVIGENCIA', $vigencia);
        $newRequest = new DTO\Request\Filter($filterCriteria, '', '', '');

        $this->dataSourceName = "obsctrlexped";
        $this->parameterBagColumns = array(
            'NRORG', 'CDCENTRAL', 'CDFILIAL', 'CDSERVFILI', 'CDCLIENTE',
            'CDDESTFILI', 'NRDIASEMANA', 'DTVIGENCIA'
        );

        $this->find($newRequest, $response);
        return $response->getDataSets()[0]->getRows();
    }
    
    public function salvaNovaObservacao(DTO\Request\Row $request, DTO\Response $response){
       try {
            $this->dataSourceName = "obsctrlexped";
            $row = $request->getRow();
            
            if ($row["__is_new"]) {
                $this->util->validateDuplicatedRecords($row, $this->dataSourceName, array("CDCENTRAL", "CDFILIAL", "CDSERVFILI", "DTVIGENCIA", "NRDIASEMANA", "NRORG"));
            }
            
            parent::save($request, $response);
            $response->addDataset(new DataSource\DataSet('RETORNO', array(true)));
        } catch (\Exception $e) {
            $response->addDataset(new DataSource\DataSet('RETORNO', array(false)));
        }
    }

    public function salvaObservacao(DTO\Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            
            $nrorg = $row['NRORG'];
            $central = $row['CDCENTRAL'];
            $filial = $row['ROW']['NMFILIAL'];
            $servico = $row['ROW']['NMSERVICO'];
            $pontoEnt = $row['ROW']['CDDESTFILI'];

            $diasSem = $row['ROW']['NMDIASEMANA'];
            if (!$diasSem) $diasSem = array('1', '2', '3', '4', '5', '6', '7', '8', '9');
            $vigencia = $row['ROW']['DTVIGENCIA'];
            $obs = $row['ROW']['DSOBSERVACAO'];

            if (!$pontoEnt) {
                $pontoEnt = $this->retornaPontoEntrega(
                    $nrorg, $servico, $filial, $response
                );
            }
            else {
                $filialPE = $row['ROW']['CDFILIAL'];
                $servicoPE = $row['ROW']['CDSERVFILI'];
                $clientePE = $row['ROW']['CDCLIENTE'];

                $pontos = array();
                foreach ($pontoEnt as $id => $valor) {
                    $pontos[] = array(
                        'CDFILIAL' => $filialPE[$id],
                        'CDSERVFILI' => $servicoPE[$id],
                        'CDCLIENTE' => $clientePE[$id],
                        'CDDESTFILI' => $valor);
                }

                $pontoEnt = $pontos;
            }

            foreach ($diasSem as $diaSemana) {
                foreach ($pontoEnt as $ponto) {
                    $obsJaCriada = $this->retornaObsPorPontoData(
                        $nrorg, $central, $ponto['CDFILIAL'],
                        $ponto['CDSERVFILI'], $ponto['CDCLIENTE'],
                        $ponto['CDDESTFILI'], $diaSemana, $vigencia,
                        new DTO\Response()
                    );

                    if (count($obsJaCriada) == 0) {
                        $dados = array(
                            'NRORG' => $nrorg,
                            '__is_new' => true,
                            '__is_new_code' => true,
                            '__table_pk' => "NROBSCTRLEXPED",
                            '__new_code_pk' => "OBSCTRLEXPED",
                            '__new_code_length' => 100,
                            'CDCENTRAL' => $central,
                            'CDFILIAL' => $ponto['CDFILIAL'],
                            'CDSERVFILI' => $ponto['CDSERVFILI'],
                            'CDCLIENTE' => $ponto['CDCLIENTE'],
                            'CDDESTFILI' => $ponto['CDDESTFILI'],
                            'NRDIASEMANA' => $diaSemana,
                            'DTVIGENCIA' => $vigencia,
                            'DSOBSERVACAO' => $obs,
                        );

                        $requestInser = new DTO\Request\Row($dados, '', '', '');
                        $dadosInserir = $this->crudService->getRowsToSave(
                            $requestInser, "obsctrlexped"
                        );
                        $this->crudService->save("obsctrlexped", $dadosInserir);
                    }
                }
            }

            $response->addDataset(new DataSource\DataSet('RETORNO', array(true)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function excluiObservacao(DTO\Request\Row $request, DTO\Response $response){
        $rows = $request->getRow();
        
        foreach ($rows[0] as $row) {
            $this->obsctrlexpedService->excluiObservacao(
                $row['NRORG'], $row['CDCENTRAL'], $row['CDFILIAL'],
                $row['CDSERVFILI'], $row['CDCLIENTE'], $row['CDDESTFILI']
            );
        }
    }

}